#include "ti86asm.inc"

.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw ShellTitle

ShellTitle:
	.db "Solubilities v1.3",0

ProgStart:
	ld a,0					;initialize the anion and cation
	ld (Anion),a			;number @ 0
	ld (Cation),a
	call _runindicoff		;turn the run indicator off

MainLoop:
	call DrawScreen			;call routine to draw everything to the screen
NoRedraw:
	call _getkey			;call routine to get key presses
	cp k7					;subtract a-k7
	jp z,Cup				;is zero flag set(a=k7)? if so scroll Cations up
	cp k4					;if 4 is pressed scroll cations down
	jp z,Cdown
	cp k8
	jp z,Aup
	cp k5
	jp z,Adown
	cp kExit				;if exit is pressed then exit the program
	jp z,end
	jp NoRedraw				;if none of the specified keys are pressed go back to top

end:
	call _clrLCD
	ret						; end program

Cup:
	ld a,(Cation)			;load Cation # into register a
	cp 0					;if it's 0 then don't scroll up anymore
	jp z,NoRedraw
	dec a					;if not then scroll up
	ld (Cation),a
	jp MainLoop				;go back to main loop

Cdown:
	ld a,(Cation)			;same as above, but down
	cp 22
	jp z,NoRedraw
	inc a
	ld (Cation),a
	jp MainLoop

Aup:						
	ld a,(Anion)			;same as above but for the anion list
	cp 0
	jp z,NoRedraw
	dec a
	ld (Anion),a
	jp MainLoop

Adown:
	ld a,(Anion)
	cp 15
	jp z,NoRedraw
	inc a
	ld (Anion),a
	jp MainLoop

DrawScreen:					;routine that calls the other routines that
	call DrawBack			;draw everything to the LCD
	call PutCations
	call PutAnions
	call PutSoluble
	call PutCatForm
	call PutAniForm
	ret

DrawBack:					;draws the background
	ld de,$fc10				;loads address for LCD into de
	ld hl,BackGround		;background points to hl
	ld bc,$3EA				;how many times to loop
	ldir 					;loop until bc = 0 while loading hl into de
	ret

PutAnions:	
	ld a,(Anion)			;load anion number into register a
	ld h,a					;these next few commands multiply anion by 11
	ld l,11
	call _HtimesL
	ld e,l
	ld d,0
	ld hl,Anions			;hl points to anion list
	add hl,de				;add offset to list
	ld b,7					;initialize loop counter
	ld e,40					;number for _penCol	
PutALoop:
	ld a,7					;these command subtract b from 7 and multiply it by 8
	sub b					;7 - b = a
	add a,a					;a*2
	add a,a					;a*4
	add a,a					;a*8
	ld d,a					
	ld (_penCol),de			;load e(40) into _penCol and d into _penRow
	call _vputs				;ROM call to put the text on screen
	djnz PutALoop			;decrement b until 0, if not 0 jump to PutALoop
	ret

PutCations:					;same as PutAnion routine but for the Cation list

	ld a,(Cation)
	ld h,a
	ld l,11
	call _HtimesL
	ld e,l
	ld d,0
	ld hl,Cations
	add hl,de
	ld b,7
	ld e,1
PutCLoop:
	ld a,7
	sub b
	add a,a
	add a,a
	add a,a
	ld d,a
	ld (_penCol),de
	call _vputs
	djnz PutCLoop
	ret

PutSoluble:					;this routine finds the corresponding number on the table
							;and displays the appropriate message
	ld a,(Cation)
	ld l,a
	ld h,0			
	add hl,hl					;multiply hl*16
	add hl,hl
	add hl,hl
	add hl,hl
	ld d,h
	ld e,l
	ld hl,Table
	add hl,de				;add offset for correct row(rows are 16 bytes long)
	ld a,(Anion)
	ld e,a
	ld d,0
	add hl,de				;add offset for correct column
	ld a,(hl)				;load number from Table into a
	ld hl,Solubles			;hl points to text for solubilities
	ld b,a					;multiply a by 14
	add a,a
	add a,a
	add a,b
	add a,b
	add a,b
	add a,a
	ld e,a
	add hl,de				;add offset for text to hl
Back2Sol:					;place where to jump back to after loading the right
	ld a,78					;phrase into hl
	ld (_penCol),a			;load 78 into _penCol	
	ld a,24
	ld (_penRow),a			;load 24 into _penRow
	call _vputs				;draw to LCD
	ret

PutCatForm:
	ld a,(Cation)
	ld b,a
	add a,a
	add a,a
	add a,b
	add a,b
	add a,b
	ld hl,CatForm
	ld e,a
	ld d,0
	add hl,de
	ld a,1
	ld (_penCol),a
	ld a,56
	ld (_penRow),a
	call _vputs
	ret

PutAniForm:
	ld a,(Anion)
	ld b,a
	add a,a
	add a,a
	add a,a
	add a,b
	ld hl,AniForm
	ld e,a
	ld d,0
	add hl,de
	ld a,40
	ld (_penCol),a
	ld a,56
	ld (_penRow),a
	call _vputs
	ret


Cation:
	.db 0
	
Anion:
	.db 0

Cations:
	.db "          ",0
	.db "          ",0
	.db "          ",0
	.db "Aluminum  ",0
	.db "Ammonium  ",0
	.db "Antimony  ",0
	.db "Arsenic   ",0
	.db "Barium    ",0
	.db "Bismuth   ",0
	.db "Cadmium   ",0
	.db "Calcium   ",0
	.db "Chromium  ",0
	.db "Cobalt    ",0
	.db "Copper    ",0
	.db "Iron III  ",0
	.db "Iron II   ",0
	.db "Lead      ",0
	.db "Magnesium ",0
	.db "Mercury II",0
	.db "Mercury I ",0
	.db "Nickel    ",0
	.db "Potassium ",0
	.db "Silver    ",0
	.db "Sodium    ",0
	.db "Strontium ",0
	.db "Zinc      ",0
	.db "          ",0
	.db "          ",0
	.db "          ",0

CatForm:
	.db "Al 3+ ",0
	.db "NH4 + ",0
	.db "Sb 3+ ",0
	.db "As 3+ ",0
	.db "Ba ",18,"+ ",0
	.db "Bi 3+ ",0
	.db "Cd ",18,"+ ",0
	.db "Ca ",18,"+ ",0
	.db "Cr 3+ ",0
	.db "Co ",18,"+ ",0
	.db "Cu ",18,"+ ",0
	.db "Fe 3+ ",0
	.db "Fe ",18,"+ ",0
	.db "Pb ",18,"+ ",0
	.db "Mg ",18,"+ ",0
	.db "Hg ",18,"+ ",0
	.db "Hg2 ",18,"+",0
	.db "Ni ",18,"+ ",0
	.db "K +   ",0
	.db "Ag +  ",0
	.db "Na +  ",0
	.db "Sr ",18,"+ ",0
	.db "Zn ",18,"+ ",0


Anions:
	.db "          ",0
	.db "          ",0
	.db "          ",0
	.db "Acetate   ",0
	.db "Bromide   ",0
	.db "Carbonate ",0
	.db "Chlorate  ",0
	.db "Chloride  ",0
	.db "Chromate  ",0
	.db "Hydroxide ",0
	.db "Iodide    ",0
	.db "Nitrate   ",0
	.db "Oxide     ",0
	.db "Oxalate   ",0
	.db "Phosphate ",0
	.db "Silicate  ",0
	.db "Sulfate   ",0
	.db "Sulfide   ",0
	.db "Sulfite   ",0
	.db "          ",0
	.db "          ",0
	.db "          ",0

AniForm:
	.db "C2H3O2 ",26,0
	.db "Br ",26,"    ",0
	.db "CO3 ",18,26,"  ",0
	.db "ClO3 ",26,"  ",0
	.db "Cl ",26,"    ",0
	.db "CrO4 ",18,26," ",0
	.db "OH ",26,"    ",0
	.db "I ",26,"     ",0
	.db "NO3 ",26,"   ",0
	.db "O ",18,26,"    ",0
	.db "C2O4 ",18,26," ",0
	.db "PO4 3",26,"  ",0
	.db "SiO3 ",18,26," ",0
	.db "SO4 ",18,26,"  ",0
	.db "S ",18,26,"    ",0
	.db "SO3 ",18,26,"  ",0

Table:
	.db $0,$0,$1,$0,$0,$1,$2,$0,$0,$2,$2,$2,$2,$0,$3,$1
	.db $0,$0,$0,$0,$0,$0,$1,$0,$0,$1,$4,$0,$1,$0,$0,$0
	.db $1,$3,$1,$1,$0,$1,$1,$3,$1,$4,$2,$1,$1,$3,$3,$1
	.db $1,$3,$1,$1,$3,$1,$1,$0,$1,$4,$1,$1,$1,$1,$2,$1
	.db $0,$0,$2,$0,$0,$2,$0,$0,$0,$0,$2,$2,$0,$2,$3,$2
	.db $2,$3,$1,$1,$3,$1,$3,$2,$3,$2,$3,$2,$2,$3,$2,$1
	.db $0,$0,$2,$0,$0,$2,$2,$0,$0,$2,$2,$2,$2,$0,$2,$4
	.db $0,$0,$2,$0,$0,$0,$2,$0,$0,$2,$2,$2,$2,$2,$2,$2
	.db $0,$0,$1,$1,$0,$1,$2,$2,$0,$2,$0,$4,$1,$0,$2,$2
	.db $0,$0,$2,$0,$0,$2,$2,$0,$0,$2,$2,$2,$2,$0,$2,$2
	.db $0,$0,$2,$0,$0,$0,$2,$1,$0,$2,$2,$2,$1,$0,$2,$1
	.db $1,$0,$1,$1,$0,$0,$2,$1,$0,$2,$0,$2,$1,$0,$2,$1
	.db $0,$0,$2,$1,$0,$2,$2,$0,$0,$2,$2,$2,$2,$0,$2,$2
	.db $0,$2,$2,$0,$4,$2,$2,$2,$0,$2,$2,$2,$2,$2,$2,$2
	.db $0,$0,$2,$0,$0,$0,$2,$0,$0,$2,$2,$2,$2,$0,$3,$0
	.db $0,$4,$2,$0,$0,$4,$1,$2,$0,$2,$2,$1,$1,$3,$2,$1
	.db $2,$2,$2,$0,$2,$2,$1,$2,$3,$2,$2,$1,$1,$2,$2,$1
	.db $0,$0,$2,$2,$0,$1,$2,$0,$0,$2,$2,$2,$1,$0,$2,$2
	.db $0,$0,$0,$0,$0,$0,$0,$0,$0,$3,$0,$0,$0,$0,$0,$0
	.db $4,$2,$2,$0,$2,$2,$1,$2,$0,$2,$2,$2,$1,$2,$2,$2
	.db $0,$0,$0,$0,$0,$0,$0,$0,$0,$3,$0,$0,$0,$0,$0,$0
	.db $0,$0,$2,$0,$0,$2,$2,$0,$0,$2,$2,$2,$2,$2,$2,$2
	.db $0,$0,$2,$0,$0,$2,$2,$0,$0,$2,$2,$2,$2,$0,$2,$2

Solubles:
	.db "Soluble      ",0
	.db "Doesn't Exist",0
	.db "Insoluble    ",0
	.db "Decomposes   ",0
	.db "Partially    ",0

BackGround:
;
; ASM code generated from:sol.bmp
; Created with BMP2ASM
;
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000011,%00000100,%00010001,%01010100,%10000000,%00000001,%00001100
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000100,%00100101,%01011000,%01000110,%00010011,%00101011,%00000010
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000010,%01010101,%01010101,%01010100,%10101010,%00101001,%00000100
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001,%01010101,%01010101,%01010100,%10110001,%00101001,%00000010
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000110,%00100101,%11011001,%01010010,%10011011,%00010011,%10101100
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111110
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000001
  .db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000,%00001000,%00000000,%00000000,%00000000,%00000000,%00000000,%00000000
  .db %11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111111,%11111000


.end
